use std::{error::Error, io::Cursor};
use actix_web::web::Bytes;
use log::warn;
use image::Rgba;
use imageproc::drawing::{draw_text_mut, draw_filled_rect_mut};
use imageproc::rect::Rect;
use ab_glyph::{Font, FontArc, PxScale};

use crate::core::get_shout_by_id;

pub async fn generate_overlay<'a>(shout_id: &'a str, filedata: Bytes) -> Result<Bytes, Box<dyn Error>> {
    // Получаем shout из GraphQL
    let shout_id_int = shout_id.parse::<i32>().unwrap_or(0);
    match get_shout_by_id(shout_id_int).await {
        Ok(shout) => {
            // Преобразуем Bytes в ImageBuffer
            let img = image::load_from_memory(&filedata)?;
            let mut img = img.to_rgba8();

            // Загружаем шрифт
            let font_vec = Vec::from(include_bytes!("Muller-Regular.woff2") as &[u8]);
            let font = FontArc::try_from_vec(font_vec).unwrap();

            // Получаем размеры изображения
            let (img_width, img_height) = img.dimensions();
            let max_text_width = (img_width as f32) * 0.8;
            let max_text_height = (img_height as f32) * 0.8;

            // Начальный масштаб
            let mut scale: f32 = 24.0;
            let text_length = shout.title.chars().count() as f32;
            let mut text_width = scale * text_length;
            let text_height = scale;

            // Регулируем масштаб, пока текст не впишется в 80% от размеров изображения
            while text_width > max_text_width || text_height > max_text_height {
                scale -= 1.0;
                if scale <= 0.0 {
                    break;
                }
                text_width = scale * text_length;
                // text_height остается равным scale
            }

            // Рассчёт позиции текста для центрирования
            let x = ((img_width as f32 - text_width) / 2.0).ceil() as i32;
            let y = ((img_height as f32 - text_height) / 2.0).ceil() as i32;

            // Задаём отступы для подложки
            let padding_x = 10;
            let padding_y = 5;

            // Определяем размеры подложки
            let rect_width = text_width.ceil() as u32 + (2 * padding_x);
            let rect_height = text_height.ceil() as u32 + (2 * padding_y);

            // Определяем координаты подложки
            let rect_x = x - padding_x as i32;
            let rect_y = y - padding_y as i32;

            // Создаём прямоугольник
            let rect = Rect::at(rect_x, rect_y).of_size(rect_width, rect_height);

            // Задаём цвет подложки (полупрозрачный серый)
            let background_color = Rgba([128u8, 128u8, 128u8, 128u8]); // RGBA: серый с прозрачностью 50%

            // Рисуем подложку
            draw_filled_rect_mut(&mut img, rect, background_color);

            // Рисуем текст поверх подложки
            let scaled_font = font.as_scaled(scale).font;
            draw_text_mut(
                &mut img,
                Rgba([255u8, 255u8, 255u8, 255u8]), // Белый цвет текста
                x,
                y,
                PxScale::from(scale),
                &scaled_font,
                &shout.title,
            );

            // Преобразуем ImageBuffer обратно в Bytes
            let mut buffer = Vec::new();
            img.write_to(&mut Cursor::new(&mut buffer), image::ImageFormat::Png)?;

            Ok(Bytes::from(buffer))
        },
        Err(e) => {
            warn!("Error getting shout: {}", e);
            Ok(filedata)
        }
    }
}
